# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""



def execute(a, b):
	"""
		please input your code below
	"""
	print("this is job one, alfred yang")
